package livecld;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.gef.EditPart;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.progress.UIJob;

public class EditPartRefreshJob extends UIJob {
	private EditPart refreshRootPart = null;
	private boolean refreshChildren = false;
	
	public EditPartRefreshJob(EditPart editPart) {
		this(editPart, false);
	}
	
	public EditPartRefreshJob(EditPart editPart, boolean refreshChildren) {
		super(Display.getDefault(), "Refresh Diagram");
		
		this.refreshRootPart = editPart;
		this.refreshChildren = refreshChildren;
		
		//set high priority
		this.setPriority(INTERACTIVE);
	}
	
	@Override
	public IStatus runInUIThread(IProgressMonitor arg0) {
		if (refreshRootPart == null)
			return Status.CANCEL_STATUS;
		
		refreshRootPart.refresh();
		
		if (refreshChildren)
			refreshChildren(refreshRootPart);
		
		return Status.OK_STATUS;
	}

	
	private void refreshChildren(EditPart part) {
		for (Object child : part.getChildren()) {
			EditPart childPart = (EditPart) child;
			childPart.refresh();
			
			refreshChildren(childPart);
		}
	}
	
}
