package livecld;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.core.Flags;
import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.ui.ISharedImages;
import org.eclipse.jdt.ui.JavaElementImageDescriptor;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;

public class IconProvider {

	private static IconProvider instance = new IconProvider();

	public static IconProvider getInstance() {
		return instance;
	}

	private Map<String, Map<Integer, Image>> imageCache = new HashMap<String, Map<Integer, Image>>();

	private IconProvider() {

	}

	private static Point SIZE = new Point(22, 16);

	public Image getIcon(IType type) {
		try {
			int flags = type.getFlags();
			int adornments = getGeneralAdornments(flags);

			// synchronized flag might be set
			if (Flags.isSynchronized(flags)) {
				// delete SYNCHRONIZED flag if set
				adornments ^= JavaElementImageDescriptor.SYNCHRONIZED;
			}

			String key = null;

			if (type.isClass()) {
				if (!type.isMember()) {
					key = ISharedImages.IMG_OBJS_CLASS;
				} else {
					if (Flags.isPrivate(flags))
						key = ISharedImages.IMG_OBJS_INNER_CLASS_PRIVATE;
					else if (Flags.isProtected(flags))
						key = ISharedImages.IMG_OBJS_INNER_CLASS_PROTECTED;
					else if (Flags.isPublic(flags))
						key = ISharedImages.IMG_OBJS_INNER_CLASS_PUBLIC;
					else
						key = ISharedImages.IMG_OBJS_INNER_CLASS_DEFAULT;
				}
			} else if (type.isInterface()) {
				if (!type.isMember()) {
					key = ISharedImages.IMG_OBJS_INTERFACE;
				} else {
					if (Flags.isPrivate(flags))
						key = ISharedImages.IMG_OBJS_INNER_INTERFACE_PRIVATE;
					else if (Flags.isProtected(flags))
						key = ISharedImages.IMG_OBJS_INNER_INTERFACE_PROTECTED;
					else if (Flags.isPublic(flags))
						key = ISharedImages.IMG_OBJS_INNER_INTERFACE_PUBLIC;
					else
						key = ISharedImages.IMG_OBJS_INNER_INTERFACE_DEFAULT;

				}
			} else if (type.isEnum()) {
				if (!type.isMember()) {
					key = ISharedImages.IMG_OBJS_ENUM;
				} else {
					if (Flags.isPrivate(flags))
						key = ISharedImages.IMG_OBJS_ENUM_PRIVATE;
					else if (Flags.isProtected(flags))
						key = ISharedImages.IMG_OBJS_ENUM_PROTECTED;
					else if (Flags.isPublic(flags))
						key = ISharedImages.IMG_OBJS_ENUM;
					else
						key = ISharedImages.IMG_OBJS_ENUM_DEFAULT;
				}
			}


			return getImageFromCache(key, adornments);
		} catch (JavaModelException e) {
			e.printStackTrace();
		}

		throw new RuntimeException("Image could not be created");
	}

	//TODO member visibility recheck!
	
	private Image getImageFromCache(String imageKey, Integer adornments) {
		if (imageCache.containsKey(imageKey)) {
			Map<Integer, Image> decoratedImageMap = imageCache.get(imageKey);
			
			if (!decoratedImageMap.containsKey(adornments)) {
				ImageDescriptor undecoratedImage = JavaUI.getSharedImages().getImageDescriptor(imageKey);
				Image image = new JavaElementImageDescriptor(undecoratedImage, adornments, SIZE).createImage();
				decoratedImageMap.put(adornments, image);
			}
		} else {
			Map<Integer, Image> decoratedImageMap = new HashMap<Integer, Image>();
			imageCache.put(imageKey, decoratedImageMap);
			
			ImageDescriptor undecoratedImage = JavaUI.getSharedImages().getImageDescriptor(imageKey);
			Image image = new JavaElementImageDescriptor(undecoratedImage, adornments, SIZE).createImage();
			decoratedImageMap.put(adornments, image);
		}
		
		return imageCache.get(imageKey).get(adornments);
	}

	public Image getIcon(IMethod method) {
		try {
			int flags = method.getFlags();
			int adornments = getGeneralAdornments(flags);

			String key = null;

			if (Flags.isPrivate(flags))
				key = ISharedImages.IMG_OBJS_PRIVATE;
			else if (Flags.isProtected(flags))
				key = ISharedImages.IMG_OBJS_PROTECTED;
			else if (Flags.isPublic(flags))
				key = ISharedImages.IMG_OBJS_PUBLIC;
			else if (Flags.isPackageDefault(flags))
				key = ISharedImages.IMG_OBJS_DEFAULT;

			if (method.isConstructor())
				adornments |= JavaElementImageDescriptor.CONSTRUCTOR;

			return getImageFromCache(key, adornments);
		} catch (JavaModelException e) {
			e.printStackTrace();
		}

		throw new RuntimeException("Image could not be created");
	}

	private int getGeneralAdornments(int flags) {
		int adornments = 0;

		if (Flags.isAbstract(flags))
			adornments |= JavaElementImageDescriptor.ABSTRACT;
		if (Flags.isFinal(flags))
			adornments |= JavaElementImageDescriptor.FINAL;
		if (Flags.isSynchronized(flags))
			adornments |= JavaElementImageDescriptor.SYNCHRONIZED;
		if (Flags.isStatic(flags))
			adornments |= JavaElementImageDescriptor.STATIC;
		if (Flags.isDeprecated(flags))
			adornments |= JavaElementImageDescriptor.DEPRECATED;

		return adornments;
	}

	public Image getIcon(IField field) {
		try {
			int flags = field.getFlags();
			int adornments = getGeneralAdornments(flags);

			String key = null;

			if (!field.isEnumConstant()) {
				if (Flags.isPrivate(flags))
					key = ISharedImages.IMG_FIELD_PRIVATE;
				else if (Flags.isProtected(flags))
					key = ISharedImages.IMG_FIELD_PROTECTED;
				else if (Flags.isPublic(flags))
					key = ISharedImages.IMG_FIELD_PUBLIC;
				else if (Flags.isPackageDefault(flags))
					key = ISharedImages.IMG_FIELD_DEFAULT;
			} else {
				// field is enum constant
				key = ISharedImages.IMG_FIELD_PUBLIC;
				adornments |= JavaElementImageDescriptor.FINAL;
				adornments |= JavaElementImageDescriptor.STATIC;
			}

			return getImageFromCache(key, adornments);
		} catch (JavaModelException e) {
			e.printStackTrace();
		}

		throw new RuntimeException("Image could not be created");
	}

}
