package livecld.actions;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jdt.core.ISourceManipulation;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.action.Action;
import org.eclipse.ui.IEditorPart;

public class DeleteElementAction extends Action {
	private List<? extends ISourceManipulation> elements;
	
	
	public DeleteElementAction(ISourceManipulation ...elements) {
		ArrayList<ISourceManipulation> list = new ArrayList<ISourceManipulation>(elements.length);
		
		for (ISourceManipulation e : elements) {
			list.add(e);
		}
		
		init(list);		
	}
	
	public DeleteElementAction(List<? extends ISourceManipulation> elements) {
		init(elements);
	}
	
	private void init(List<? extends ISourceManipulation> elements) {
		this.elements = elements;
		
		setText("Delete");
		
		if (elements == null || elements.size() < 1)
			setEnabled(false);
	}
	
	@Override
	public void run() {
		for (ISourceManipulation element : elements) {
			
			try {
				if (element instanceof IType) {
					IType type = (IType) element;
					
					
					if (type.isMember())
						type.delete(false, new NullProgressMonitor());
					else {
						IEditorPart part = JavaUI.openInEditor(type);
						part.getSite().getPage().closeEditor(part, false);
						
						type.getCompilationUnit().delete(true, new NullProgressMonitor());
					}
						
				} else {
					element.delete(false, new NullProgressMonitor());
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
}
