package livecld.controller;

import livecld.model.DiagrammRootModel;
import livecld.model.GeneralizationConnectionModel;
import livecld.model.HierarchyNodeModel;
import livecld.model.MessageModel;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;


public class ClassDiagrammPartFactory implements EditPartFactory {

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart, java.lang.Object)
	 */
	public EditPart createEditPart(EditPart context, Object modelElement) {
		// get EditPart for model element
		EditPart part = getPartForElement(modelElement);
		// store model element in EditPart
		part.setModel(modelElement);
		return part;
	}
	
	
	/**
	 * Maps an object to an EditPart. 
	 */
	private EditPart getPartForElement(Object model) {
//		System.out.println("Factory: " + model.getClass());
		
		if (model instanceof DiagrammRootModel) {
			return new DiagrammRootEditPart();
		}
		else if (model instanceof HierarchyNodeModel) {
			return new HierarchyNodeEditPart();
		}
		else if (model instanceof IType) {
			return new TypeEditPart();
		}
		else if (model instanceof IField) {
			return new FieldEditPart();
		}	
		else if (model instanceof IMethod) {
			return new MethodEditPart();
		}
		else if (model instanceof GeneralizationConnectionModel) {
			return new GeneralizationConnectionEditPart();
		} else if (model instanceof MessageModel)
			return new MessageEditPart();
		
		throw new RuntimeException("Can't create part for model element: " + model.getClass().getName());
	}

}
