package livecld.controller;

import java.util.ArrayList;
import java.util.List;

import livecld.EditPartRefreshJob;
import livecld.IconProvider;
import livecld.TextProvider;
import livecld.javamodel.JavaModelListener;
import livecld.javamodel.filter.JavaElementFilter;
import livecld.model.HierarchyNodeModel;
import livecld.view.BottomBorderAnchor;
import livecld.view.TopCenterAnchor;
import livecld.view.TypeFigure;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.jdt.core.ElementChangedEvent;
import org.eclipse.jdt.core.IElementChangedListener;
import org.eclipse.jdt.core.IJavaElementDelta;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;


public class TypeEditPart extends JavaElementEditPart implements NodeEditPart, IElementChangedListener {
	
	private Object dummyFieldsModel = new Object();
	private Object dummyMethodsModel = new Object();
	
	@Override
	public void activate() {
		if (!isActive()) {
			super.activate();
			JavaModelListener.getListener().addChangedListener(this, new JavaElementFilter(getCastedModel()));
		}
	}
	
	@Override
	public void deactivate() {
		if (isActive()) {
			super.deactivate();
			JavaModelListener.getListener().addChangedListener(this);
		}
	}
	
	@Override
	protected IFigure createFigure() {
		TypeFigure f = new TypeFigure();
		return f;
	}
	
	
	@Override
	protected void refreshVisuals() {
		TypeFigure f = (TypeFigure) getFigure();
		IType model = getCastedModel();
		f.setText(TextProvider.getInstance().getText(model));
		f.setIcon(IconProvider.getInstance().getIcon(model));
	}
	
	
	public IType getCastedModel() {
		return (IType)getModel();
	}
	
	
	@Override
	public List getModelChildren() {
 		List<Object> list = new ArrayList<Object>();
 		
 		try {
 			boolean showBinary = ((DiagrammRootEditPart)getRoot().getContents()).isShowBinaryMembers();
 			
 			if (!getCastedModel().isBinary() || showBinary) {
				if (getCastedModel().getFields().length > 0)
					list.add(dummyFieldsModel);
				if (getCastedModel().getMethods().length > 0)
					list.add(dummyMethodsModel);		
 			}
		} catch (JavaModelException e) {
			e.printStackTrace();
		}

 		return list;
	}
	
	@Override
	protected EditPart createChild(Object model) {
		EditPart part = null;
		
		if (model == dummyFieldsModel) {
			part = new FieldCompartmentEditPart();
		} else if (model == dummyMethodsModel) {
			part = new MethodCompartmentEditPart();
		}
		
		part.setModel(model);
		return part;
	}
	
	@Override
	public IFigure getContentPane() {
		return ((TypeFigure)getFigure()).getContentPane();
	}

	
	public HierarchyNodeModel getHierarchyNodeModel() {
		//HierarchyNodeEditPart -> HierarchyCompartmentEditPart -> TypeEditPart
		HierarchyNodeEditPart node = (HierarchyNodeEditPart) getParent().getParent();
		return node.getCastedModel();
	}


	@Override
	public List getModelSourceConnections() {
		return getHierarchyNodeModel().getSourceConnections();
	}
	
	@Override
	public List getModelTargetConnections() {
		return getHierarchyNodeModel().getTargetConnections();
	}
	
	
	
	private ConnectionAnchor sourceAnchor;
	private ConnectionAnchor targetAnchor;
	
	
	private ConnectionAnchor getSourceAnchor() {
		if (sourceAnchor == null) {
			sourceAnchor = new TopCenterAnchor(getFigure());
		}
		return sourceAnchor;
	}
	
	private ConnectionAnchor getTargetAnchor() {
		if (targetAnchor == null) {
			targetAnchor = new BottomBorderAnchor(getFigure());
		}
		return targetAnchor;
	}
	

	
	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
		return getSourceAnchor();
	}


	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
		return getSourceAnchor();
	}


	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {
		return getTargetAnchor();
	}


	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
		return getTargetAnchor();
	}
	
	
	public void elementChanged(ElementChangedEvent evt) {
		IJavaElementDelta delta = evt.getDelta();

		
//		System.out.println("Type changed: " + delta);
		
		//refresh editpart tree if fields/methods have been added/removed
		if (delta.getAddedChildren().length > 0 || delta.getRemovedChildren().length > 0 || delta.getChangedChildren().length > 0 ) {
			new EditPartRefreshJob(this, true).schedule();
		}
//		if (delta.getKind() == IJavaElementDelta.CHANGED) {
//			if ((delta.getFlags() & (IJavaElementDelta.F_MODIFIERS)) != 0) {
//				new EditPartRefreshJob(this).schedule();
//			}
//		}
	}
	
}
