package livecld.javamodel;

import java.util.LinkedList;

import livecld.javamodel.filter.JavaModelFilter;
import livecld.javamodel.filter.NullFilter;

import org.eclipse.jdt.core.ElementChangedEvent;
import org.eclipse.jdt.core.IElementChangedListener;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaElementDelta;
import org.eclipse.jdt.core.JavaCore;


public class JavaModelListener implements IElementChangedListener {

	private static JavaModelListener instance = new JavaModelListener();
	
	public static JavaModelListener getListener() {
		return instance;
	}


	private JavaModelListener() {
		JavaCore.addElementChangedListener(this);
	}

	public void elementChanged(ElementChangedEvent event) {
		IJavaElementDelta delta = event.getDelta();
		
		//TODO debug
//		traverseAndPrint(delta);
		
		if (delta != null) {
			traverseDelta(delta, event.getType());
		}
	}
	
	
	private LinkedList<Entry> listeners = new LinkedList<Entry>();
	
	public void addChangedListener(IElementChangedListener listener, JavaModelFilter filter) {
		if (filter == null) {
			filter = new NullFilter();
		}
		
		listeners.add(new Entry(listener,filter));
	}
	
	
	public void addChangedListener(IElementChangedListener listener) {
		listeners.remove(new Entry(listener));
	}
		

	
	
	private void traverseDelta(IJavaElementDelta delta, int type) {
		IJavaElement element = delta.getElement();
		
		
		ElementChangedEvent evt = null;
		
		for (Entry entry : listeners) {
			if (entry.getFilter().accept(element)) {
				if (evt == null)
					evt = new ElementChangedEvent(delta, type);
				
				entry.getListener().elementChanged(evt);
			}
		}
		
		for (IJavaElementDelta child : delta.getAffectedChildren()) {
			traverseDelta(child, type);
		}
	}
	


	/**
	 * Debug method
	 * @param delta
	 */
	@SuppressWarnings("unused")
	private void traverseAndPrint(IJavaElementDelta delta) {
        switch (delta.getKind()) {
            case IJavaElementDelta.ADDED:
                System.out.println(delta.getElement().getClass() + " was added");
                break;
            case IJavaElementDelta.REMOVED:
                System.out.println(delta.getElement().getClass() + " was removed");
                break;
            case IJavaElementDelta.CHANGED:
                System.out.println(delta.getElement().getClass() + " was changed");
                if ((delta.getFlags() & IJavaElementDelta.F_CHILDREN) != 0) {
                    System.out.println("The change was in its children");
                }
                if ((delta.getFlags() & IJavaElementDelta.F_CONTENT) != 0) {
                    System.out.println("The change was in its content");
                }
                /* Others flags can also be checked */
                break;
        }
        IJavaElementDelta[] children = delta.getAffectedChildren();
        for (int i = 0; i < children.length; i++) {
            traverseAndPrint(children[i]);
        }
    }

    
    
    
    private class Entry {
    	private JavaModelFilter filter;
    	private IElementChangedListener listener;
    	
		public Entry(IElementChangedListener listener, JavaModelFilter filter) {
			this.filter = filter;
			this.listener = listener;
		}
		
		public Entry(IElementChangedListener listener) {
			this(listener,null);
		}
   
		
		public JavaModelFilter getFilter() {
			return filter;
		}
		
		public IElementChangedListener getListener() {
			return listener;
		}
		
		@Override
		public boolean equals(Object object) {
			if (object instanceof Entry) {
				Entry e = (Entry) object;
				return listener.equals(e.getListener());
			}
			
			return false;
		}
    	
    }
    
}
